require 'spec_helper'
require 'rspec/expectations'

describe Puppet::Type.type(:equallogic_volume_chap_user).provider(:equallogic_volume_chap_user) do
  let(:resource) {Puppet::Type.type(:equallogic_volume_chap_user).new(
      {
          :name              => 'TestUser',
          :ensure            => 'present',
          :chap_password     => 'TestPassword',
          :state             => 'enable',
          :require           => 'Equallogic_volume["TestVol"]'
      }
  )}
  let(:provider) {resource.provider}

  before :each do
    @fixture_dir = File.join(Dir.pwd,'spec','fixtures')
    Puppet::Equallogic::Util.stub(:get_transport).and_return({
                                                                 :host => '172.17.4.10',
                                                                 :user => 'grpadmin',
                                                                 :password => 'password'
                                                             })
  end

  describe 'exists?' do
    context 'when chap user exists' do
      context 'when password is not matching' do
        it 'raises error' do
          provider.stub(:get_chap_user_list).and_return(File.read(File.join(@fixture_dir,'show_chap_user_present')))
          expect{provider.exists?}.to raise_error('Provided password does not match existing password for chap user TestUser')
        end
      end
      context 'when password is matching' do
        it 'returns true' do
          provider.stub(:get_chap_user_list).and_return(File.read(File.join(@fixture_dir,'show_chap_user_match')))
          expect(provider.exists?).to be true
        end
      end
    end
    context 'when chap user does not exist' do
      it 'returns false' do
        provider.stub(:get_chap_user_list).and_return(File.read(File.join(@fixture_dir,'show_chap_user')))
        expect(provider.exists?).to be false
      end
    end
  end

  describe 'get_chap_user_info' do
    it 'gets user info for a volume' do
      parsed_users = [
          {:name=>"Windows", :chap_password=>"123456789123"},
          {:name=>"quorum", :chap_password=>"123456789123"},
          {:name=>"admin", :chap_password=>"passwordpassword"},
          {:name=>"chapuser1", :chap_password=>"chapusersecret"},
          {:name=>"chapuser2", :chap_password=>"chapusersecret"},
          {:name=>"chapuser", :chap_password=>"chapsecret"},
          {:name=>"chapuser10000", :chap_password=>"chapsecret1000"},
          {:name=>"bfschapuser", :chap_password=>"bfschapsecret"},
          {:name=>"chapuserJDQSGT1", :chap_password=>"chapusernewpatch"},
          {:name=>"puppet", :chap_password=>"puppet"},
          {:name=>"johnk", :chap_password=>"johnk"},
          {:name=>"asm", :chap_password=>"asm"},
          {:name=>"adan-chap", :chap_password=>"P@ssw0rd"},
          {:name=>"asdf123", :chap_password=>"asdf123"},
          {:name=>"chapp1", :chap_password=>"chapp1"},
          {:name=>"chappy", :chap_password=>"chappy"},
          {:name=>"chappyx", :chap_password=>"chappy"},
          {:name=>"chappy123", :chap_password=>"chappy"},
          {:name=>"chappyxxxxx", :chap_password=>"chappy"},
          {:name=>"cchhhaa", :chap_password=>"cchhhaa"},
          {:name=>"aaaa", :chap_password=>"aaaa"},
          {:name=>"chapppppy", :chap_password=>"123123"},
          {:name=>"abc789", :chap_password=>"abc789"},
          {:name=>"abc456", :chap_password =>"abc456"},
          {:name=>"abc456f344f4f4", :chap_password=>"xxx3fs54"},
          {:name=>"abc1234567", :chap_password=>"xxxxxxx"},
          {:name=>"abc1234567890", :chap_password=>"ddt34fgfgdf"},
          {:name=>"xxx33334442", :chap_password=>"asdfasdf"},
          {:name=>"xxx33334442gwe54g4e5ege45", :chap_password=>"xxxxxxxxxxx"},
          {:name=>"uxxx", :chap_password=>"uxxx"},
          {:name=>"er4r534rf", :chap_password=>"g546455"},
          {:name=>"4feases4", :chap_password=>"rrrr"},
          {:name=>"bbbbasw4", :chap_password=>"rrr34"},
          {:name=>"aaabbb", :chap_password=>"aaabbb"},
          {:name=>"abc1239", :chap_password=>"abc1239"},
          {:name=>"aaaccc", :chap_password=>"aaaccc"},
          {:name=>"aattee", :chap_password=>"aattee"},
          {:name=>"loop", :chap_password=>"loop"},
          {:name=>"ddd", :chap_password=>"ddd"},
          {:name=>"adan-chap2", :chap_password=>"P@ssw0rd"},
          {:name=>"user", :chap_password=>"pass"},
          {:name=>"rbuser1", :chap_password=>"calvin"},
          {:name=>"logan", :chap_password=>"admin"},
          {:name=>"TestUser", :chap_password=>"P@ssw0rd"}
      ]
      provider.stub(:get_chap_user_list).and_return(File.read(File.join(@fixture_dir,'show_chap_user_present')))
      provider.get_chap_user_info.should == parsed_users
    end
  end

  describe 'create' do
    it 'creates the chap user' do
      provider.stub(:create_chap_user).and_return("CHAP user creation succeeded.\n")
      expect(provider.create).to be true
    end
  end

  describe 'destroy' do
    it 'removes the CHAP user' do
      provider.stub(:delete_chap_user).and_return("CHAP user deletion succeeded.\n")
      provider.destroy.should == :absent
    end
  end

end