require 'spec_helper'
require 'rspec/expectations'

describe Puppet::Type.type(:equallogic_volume).provider(:equallogic_volume) do
  let(:resource) {Puppet::Type.type(:equallogic_volume).new(
      {
          :name              => 'TestVol',
          :ensure            => 'present',
          :state             => 'online',
          :initsize          => '20GB',
          :thinprovision     => 'enable',
          :snapreserve       => '100%',
          :thinminreserve    => '10%',
          :thingrowthwarn    => '60%',
          :thingrowthmax     => '100%',
          :thinwarnsoftthres => '60%',
          :thinwarnhardthres => '90%',
          :multihostaccess   => 'enable',
          :poolname          => 'default',
      }
  )}
  let(:provider) {resource.provider}

  before :each do
    @fixture_dir = File.join(Dir.pwd,'spec','fixtures')
    Puppet::Equallogic::Util.stub(:get_transport).and_return({
                                                                 :host => '172.17.4.10',
                                                                 :user => 'grpadmin',
                                                                 :password => 'password'
                                                             })
  end

  describe 'get_volinfo' do
    context 'when volume exists' do
      it 'gets the volume information' do
        provider.class.any_instance.stub(:get_volume_info_for_vol).and_return(File.read(File.join(@fixture_dir,'get_volume_info_for_vol')))
        provider.get_volinfo.should == {
            :name => 'TestVol',
            :size => '20GB',
            :snapshots	=> '0',
            :status		=> 'online',
            :permissions	=> 'read-write',
            :connections	=> '0',
            :template	=> 'N',
            :hostaccess	=> 'enabled'
        }
      end
    end
    context 'when volume does not exist' do
      it 'returns an error' do
        provider.class.any_instance.stub(:get_volume_info_for_vol).and_return(File.read(File.join(@fixture_dir,'get_volume_info_for_vol_missing')))
        expect(provider.get_volinfo).to raise_error
      end
    end
  end

  describe 'exists?' do
    context 'when volume is already created' do
      it 'returns true' do
        provider.class.any_instance.stub(:get_volume_info_for_vol).and_return(File.read(File.join(@fixture_dir,'get_volume_info_for_vol')))
        expect(provider.exists?).to be true
      end
    end
    context 'when volume is not created' do
      it 'returns false' do
        provider.class.any_instance.stub(:get_volume_info_for_vol).and_return(File.read(File.join(@fixture_dir,'get_volume_info_for_vol_missing')))
        expect(provider.exists?).to be false
      end
    end
  end

  describe 'create' do
    context 'without thin provisioning' do
      let(:params) { {:thinprovision => 'disable'}}
      it 'creates a volume without thin provisioning' do
        provider.stub(:create_volume_without_thinprovision).and_return(File.read(File.join(@fixture_dir,'vol_create_nothin')))
        provider.stub(:set_multihost_access).and_return("\n")
        expect(provider.create).to be true
      end
    end

    context 'with thin provisioning' do
      let(:params) { {:thinprovision => 'enable'}}
      it 'creates a volume with thin provisioning' do
        provider.stub(:create_volume_with_thinprovision).and_return(File.read(File.join(@fixture_dir,'vol_create_thin')))
        provider.stub(:set_multihost_access).and_return("\n")
        expect(provider.create).to be true
      end
    end
  end

  describe 'destroy' do
    it 'destroys the volume' do
      provider.stub(:set_volume_offline).and_return("\n")
      provider.stub(:destroy_volume).and_return("Volume deletion succeeded.\n")
      provider.destroy.should == :absent
    end
  end

end